dataDir                   = {{data_dir}}
clientPort                = 2181
tickTime                  = 2000
maxClientCnxns            = 60
initLimit                 = 10
syncLimit                 = 5
autopurge.snapRetainCount = 3
autopurge.purgeInterval   = 1